/*******************************************************************************
* File Name: TP.c
* Version 2.0
*
* Description:
*  This file provides the API source code for the Resistive Touch component.
*
* Note:
*
********************************************************************************
* Copyright 2012-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "CyLib.h"
#include "TP.h"
#include "TP_xm.h"
#include "TP_xp.h"
#include "TP_ym.h"
#include "TP_yp.h"

uint8 TP_initVar = 0u;
static uint8 TP_measurementFlag = 0u;


/*******************************************************************************
* Function Name: TP_Init
********************************************************************************
*
* Summary:
*  Calls the Init() function of the ADC and AMux components.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void TP_Init(void) 
{
    TP_AMux_Init();
    TP_ADC_Init();
}


/*******************************************************************************
* Function Name: TP_Start
********************************************************************************
*
* Summary:
*  Calls Init() if the component has not been initialized before. Calls Enable()
*  to begin the component operation.
*
* Parameters:
*  None
*
* Return:
*  None
* 
* Global variables:
*  TP_initVar - Indicates whether the component has been
*                             initialized.
*
*******************************************************************************/
void TP_Start(void)  
{
    if(TP_initVar == 0u)
    {    
        TP_Init();
        TP_initVar = 1u;
    }
    TP_Enable();
}


/*******************************************************************************
* Function Name: TP_Stop
********************************************************************************
*
* Summary:
*  Stops the ADC and AMux components.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void TP_Stop(void)  
{
    TP_ADC_Stop();
    TP_AMux_Stop();
}


/*******************************************************************************
* Function Name: TP_Enable
********************************************************************************
*
* Summary:
*  Enables the ADC component.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void TP_Enable(void)  
{
    TP_ADC_Enable();
}


/*******************************************************************************
* Function Name: TP_ActivateX
********************************************************************************
*
* Summary: 
*  Configures the pins to measure the Y-axis.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void TP_ActivateX(void) 
{
    /* Pins configuration to measure Y. */
    CyPins_SetPinDriveMode(TP_ym_0, PIN_DM_STRONG);
    CyPins_SetPinDriveMode(TP_yp_0, PIN_DM_STRONG);
    CyPins_SetPinDriveMode(TP_xp_0, PIN_DM_ALG_HIZ);
    CyPins_SetPinDriveMode(TP_xm_0, PIN_DM_ALG_HIZ);
    
    /* Switch AMux if channel 0 is not selected. */
    if(TP_AMux_GetChannel() != TP_AMUX_XP_CHAN)
    {
        TP_AMux_Next();
    }
}


/*******************************************************************************
* Function Name: TP_ActivateY
********************************************************************************
*
* Summary: 
*  Configures the pins to measure X-axis.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void TP_ActivateY(void) 
{
    /* Pins configuration to measure X. */
    CyPins_SetPinDriveMode(TP_xm_0, PIN_DM_STRONG);
    CyPins_SetPinDriveMode(TP_xp_0, PIN_DM_STRONG);
    CyPins_SetPinDriveMode(TP_yp_0, PIN_DM_ALG_HIZ);
    CyPins_SetPinDriveMode(TP_ym_0, PIN_DM_ALG_HIZ);
    
    /* Switch AMux if channel 1 is not selected. */
    if(TP_AMux_GetChannel() == TP_AMUX_XP_CHAN)
    {
        TP_AMux_Next();
    }
    /* Switch AMux to channel 1 if AMux is disconnected. */
    else if(TP_AMux_GetChannel() == TP_AMUX_NO_CHAN)
    {
        TP_AMux_Next();
        TP_AMux_Next();
    }
    else
    {
        /* Channel 1 is already selected. */
    }
}


/*******************************************************************************
* Function Name: TP_TouchDetect
********************************************************************************
*
* Summary:
*  Determines if the screen is touched.
*
* Parameters:
*  None
*
* Return:
*  uint8: The touch state.
*  0 - untouched
*  1 - touched
*
*******************************************************************************/
uint8 TP_TouchDetect(void) 
{
    /* Configure x+ pin to detect touch. */
    CyPins_SetPinDriveMode(TP_xp_0, PIN_DM_STRONG);

    /* Add delay for signal to stabilize. */
    CyDelayUs(5u);

    CyPins_SetPinDriveMode(TP_xp_0, PIN_DM_RES_UP);
    
    /* Switch AMux if channel 0 is not selected. */
    if(TP_AMux_GetChannel() != TP_AMUX_XP_CHAN)
    {
        TP_AMux_Next();
    }

    /* Configure x-, y+ and y- to detect touch. */
    CyPins_SetPinDriveMode(TP_xm_0, PIN_DM_ALG_HIZ);
    CyPins_SetPinDriveMode(TP_yp_0, PIN_DM_ALG_HIZ);
    CyPins_SetPinDriveMode(TP_ym_0, PIN_DM_STRONG);

    /* Add delay for signal to stabilize. The delay is dependant on whether
    * the measurement has been done before touch detect or not.
    */
    if (TP_measurementFlag == 0u)
    {
        CyDelayUs(20u);
    }
    else
    {
        TP_measurementFlag = 0u;
        CyDelayUs(8u);        
    }
    /* The screen is touched if the measured value is lower than the specified
    * threshold.
    */
    return ((TP_Measure() < TP_TOUCH_THRESHOLD) ? 1u : 0u);
}


/*******************************************************************************
* Function Name: TP_Measure
********************************************************************************
*
* Summary:
*  Returns the result of the ADC conversion. Does not return until ADC
*  conversion is complete.
*
* Parameters:
*  None
*
* Return:
*  int16: the result of the ADC conversion.
*
*******************************************************************************/
int16 TP_Measure(void) 
{       
    int16 result;

    /* Trigger the ADC conversion and wait for results (blocking). */
    TP_ADC_StartConvert();
    (void)TP_ADC_IsEndConversion(TP_ADC_WAIT_FOR_RESULT);
    result = TP_ADC_GetResult16();

    /* Assert a flag indicating that the measurement has been done. */
    TP_measurementFlag = 1u;
    return (result);
}


/* [] END OF FILE */
