/*******************************************************************************
* File Name: UART_S8_PM.c
* Version 2.50
*
* Description:
*  This file provides Sleep/WakeUp APIs functionality.
*
* Note:
*
********************************************************************************
* Copyright 2008-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "UART_S8.h"


/***************************************
* Local data allocation
***************************************/

static UART_S8_BACKUP_STRUCT  UART_S8_backup =
{
    /* enableState - disabled */
    0u,
};



/*******************************************************************************
* Function Name: UART_S8_SaveConfig
********************************************************************************
*
* Summary:
*  This function saves the component nonretention control register.
*  Does not save the FIFO which is a set of nonretention registers.
*  This function is called by the UART_S8_Sleep() function.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  UART_S8_backup - modified when non-retention registers are saved.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void UART_S8_SaveConfig(void)
{
    #if(UART_S8_CONTROL_REG_REMOVED == 0u)
        UART_S8_backup.cr = UART_S8_CONTROL_REG;
    #endif /* End UART_S8_CONTROL_REG_REMOVED */
}


/*******************************************************************************
* Function Name: UART_S8_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the nonretention control register except FIFO.
*  Does not restore the FIFO which is a set of nonretention registers.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  UART_S8_backup - used when non-retention registers are restored.
*
* Reentrant:
*  No.
*
* Notes:
*  If this function is called without calling UART_S8_SaveConfig() 
*  first, the data loaded may be incorrect.
*
*******************************************************************************/
void UART_S8_RestoreConfig(void)
{
    #if(UART_S8_CONTROL_REG_REMOVED == 0u)
        UART_S8_CONTROL_REG = UART_S8_backup.cr;
    #endif /* End UART_S8_CONTROL_REG_REMOVED */
}


/*******************************************************************************
* Function Name: UART_S8_Sleep
********************************************************************************
*
* Summary:
*  This is the preferred API to prepare the component for sleep. 
*  The UART_S8_Sleep() API saves the current component state. Then it
*  calls the UART_S8_Stop() function and calls 
*  UART_S8_SaveConfig() to save the hardware configuration.
*  Call the UART_S8_Sleep() function before calling the CyPmSleep() 
*  or the CyPmHibernate() function. 
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  UART_S8_backup - modified when non-retention registers are saved.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void UART_S8_Sleep(void)
{
    #if(UART_S8_RX_ENABLED || UART_S8_HD_ENABLED)
        if((UART_S8_RXSTATUS_ACTL_REG  & UART_S8_INT_ENABLE) != 0u)
        {
            UART_S8_backup.enableState = 1u;
        }
        else
        {
            UART_S8_backup.enableState = 0u;
        }
    #else
        if((UART_S8_TXSTATUS_ACTL_REG  & UART_S8_INT_ENABLE) !=0u)
        {
            UART_S8_backup.enableState = 1u;
        }
        else
        {
            UART_S8_backup.enableState = 0u;
        }
    #endif /* End UART_S8_RX_ENABLED || UART_S8_HD_ENABLED*/

    UART_S8_Stop();
    UART_S8_SaveConfig();
}


/*******************************************************************************
* Function Name: UART_S8_Wakeup
********************************************************************************
*
* Summary:
*  This is the preferred API to restore the component to the state when 
*  UART_S8_Sleep() was called. The UART_S8_Wakeup() function
*  calls the UART_S8_RestoreConfig() function to restore the 
*  configuration. If the component was enabled before the 
*  UART_S8_Sleep() function was called, the UART_S8_Wakeup()
*  function will also re-enable the component.
*
* Parameters:
*  None.
*
* Return:
*  None.
*
* Global Variables:
*  UART_S8_backup - used when non-retention registers are restored.
*
* Reentrant:
*  No.
*
*******************************************************************************/
void UART_S8_Wakeup(void)
{
    UART_S8_RestoreConfig();
    #if( (UART_S8_RX_ENABLED) || (UART_S8_HD_ENABLED) )
        UART_S8_ClearRxBuffer();
    #endif /* End (UART_S8_RX_ENABLED) || (UART_S8_HD_ENABLED) */
    #if(UART_S8_TX_ENABLED || UART_S8_HD_ENABLED)
        UART_S8_ClearTxBuffer();
    #endif /* End UART_S8_TX_ENABLED || UART_S8_HD_ENABLED */

    if(UART_S8_backup.enableState != 0u)
    {
        UART_S8_Enable();
    }
}


/* [] END OF FILE */
