/*********************************************************************
*                SEGGER Microcontroller GmbH & Co. KG                *
*        Solutions for real time microcontroller applications        *
**********************************************************************
*                                                                    *
*        (c) 1996 - 2009  SEGGER Microcontroller GmbH & Co. KG       *
*                                                                    *
*        Internet: www.segger.com    Support:  support@segger.com    *
*                                                                    *
**********************************************************************

** emWin V5.00 - Graphical user interface for embedded applications **
emWin is protected by international copyright laws.   Knowledge of the
source code may not be used to write a similar product.  This file may
only be used in accordance with a license and should not be re-
distributed in any way. We appreciate your understanding and fairness.
----------------------------------------------------------------------
File        : LCDConf.c
Purpose     : Display controller configuration (single layer)
---------------------------END-OF-HEADER------------------------------
*/

#include <project.h>

#include "GUI.h"
#include "LCDConf_CompactColor_16.h"


/*********************************************************************
*       Layer configuration
*********************************************************************/

/* Physical display size */
#define XSIZE_PHYS          240
#define YSIZE_PHYS          320

#define VXSIZE_PHYS         240
#define VYSIZE_PHYS         320

/* Color conversion */
#define COLOR_CONVERSION    GUICC_M565

/* Display driver */
#define DISPLAY_DRIVER GUIDRV_COMPACT_COLOR_16


/*********************************************************************
*       Configuration checking
*********************************************************************/

#ifndef   VXSIZE_PHYS
  #define VXSIZE_PHYS XSIZE_PHYS
#endif
#ifndef   VYSIZE_PHYS
  #define VYSIZE_PHYS YSIZE_PHYS
#endif
#ifndef   XSIZE_PHYS
  #error Physical X size of display is not defined!
#endif
#ifndef   YSIZE_PHYS
  #error Physical Y size of display is not defined!
#endif
#ifndef   COLOR_CONVERSION
  #error Color conversion not defined!
#endif
#ifndef   DISPLAY_DRIVER
  #error No display driver defined!
#endif
#ifndef   LCD_SWAP_XY
  #define LCD_SWAP_XY 0
#endif
#if LCD_SWAP_XY
  #define LCD_XSIZE YSIZE_PHYS
  #define LCD_YSIZE XSIZE_PHYS
#else
  #define LCD_XSIZE XSIZE_PHYS
  #define LCD_YSIZE YSIZE_PHYS
#endif
#define LCD_VXSIZE LCD_XSIZE
#define LCD_VYSIZE LCD_YSIZE


/*********************************************************************
*
*           Access routines
*
*********************************************************************/

#if LCD_USE_PARALLEL_16 /* 16-bit parallel interface */
  
  /* Write multiple bytes to controller, with A0 = 1 */
  void LCD_Intf16_WriteM01(unsigned short * pData, int NumWords) {
    for (; NumWords; NumWords--) {
      CYGRAPHICS_WRITE(1, *pData++);
    }
  }
  
  /* Write multiple bytes to controller, with A0 = 0 */
  void LCD_Intf16_WriteM00(unsigned short * pData, int NumWords) {
    for (; NumWords; NumWords--) {
      CYGRAPHICS_WRITE(0, *pData++);
    }
  }
  
 /* Read multiple bytes from controller, with A0 = 1 */
  void LCD_Intf16_ReadM01(unsigned short * pData, int NumWords) {
    for (; NumWords; NumWords--) {
      *pData++ = CYGRAPHICS_READ(1);
    }
  }
#else /* 8-bit parallel interface */
  /* Write multiple bytes to controller, with A0 = 1 */
  void LCD_Intf8_WriteM01(unsigned char * pData, int NumBytes) {
    for (; NumBytes; NumBytes--) {
      CYGRAPHICS_WRITE(1, *pData++);
    }
  }

/* Write multiple bytes to controller, with A0 = 0 */
void LCD_Intf8_WriteM00(unsigned char * pData, int NumBytes) {
  for (; NumBytes; NumBytes--) {
    CYGRAPHICS_WRITE(0, *pData++);
  }
}

/* Read multiple bytes from controller, with A0 = 1 */
void LCD_Intf8_ReadM01(unsigned char * pData, int NumBytes) {
  for (; NumBytes; NumBytes--) {
    *pData++ = CYGRAPHICS_READ(1);
  }
} 
#endif


/*********************************************************************
*
*       LCD_X_InitController
*
* Purpose:
*   Initializes the display controller
*/
static void _InitController(void) 
{
    /* Set up the display controller and put it into operation. If the 
    *  display controller is not initialized by any external routine 
    *  this needs to be adapted by the customer.
    */

}


/*********************************************************************
*       Public code
*********************************************************************/

/*********************************************************************
*
*       LCD_X_Config
*
* Purpose:
*   Called during the initialization process in order to set up the
*   display driver configuration.
*   
*/

// NOTE: With respect to the vertical orientation
//#define TOUCH_ADC_TOP    262
//#define TOUCH_ADC_BOTTOM 2540
//#define TOUCH_ADC_LEFT   353
//#define TOUCH_ADC_RIGHT  2293

#define TOUCH_ADC_TOP    2260
#define TOUCH_ADC_BOTTOM  400
#define TOUCH_ADC_LEFT   2600
#define TOUCH_ADC_RIGHT   300

void LCD_X_Config(void) 
{
    /* Set display driver and color conversion */
    GUI_DEVICE_CreateAndLink(DISPLAY_DRIVER, COLOR_CONVERSION, 0, 0);
    
    /* Display driver configuration */
    LCD_SetSizeEx (0, LCD_XSIZE,   LCD_YSIZE);
    LCD_SetVSizeEx(0, LCD_VXSIZE,  LCD_VYSIZE);  
	
	#if (GUI_SUPPORT_TOUCH)
        /* Set the logical display orientation */
        GUI_TOUCH_SetOrientation((LCD_MIRROR_X << 0) | (LCD_MIRROR_Y << 1) | (LCD_SWAP_XY << 2));      

      	// Calibration (TODO: review it)
    	//GUI_TOUCH_Calibrate(GUI_COORD_X, 0, XSIZE_PHYS-1, 0, 0);    	
    	//GUI_TOUCH_Calibrate(GUI_COORD_Y, 0, YSIZE_PHYS-1, 0, 0); 

    	GUI_TOUCH_Calibrate(GUI_COORD_X, 0, XSIZE_PHYS-1, TOUCH_ADC_RIGHT, TOUCH_ADC_LEFT);    	
    	GUI_TOUCH_Calibrate(GUI_COORD_Y, 0, YSIZE_PHYS-1, TOUCH_ADC_TOP, TOUCH_ADC_BOTTOM); 

        /* Start resistive touch component */
        CYTOUCH_START();        
    #endif
}

/*********************************************************************
*
*       LCD_X_DisplayDriver
*
* Purpose:
*   This function is called by the display driver for several purposes.
*   To support the according task the routine needs to be adapted to
*   the display controller. Please note that the commands marked with
*   'optional' are not cogently required and should only be adapted if 
*   the display controller supports these features.
*
* Parameter:
*   LayerIndex - Index of layer to be configured
*   Cmd        - Please refer to the details in the switch statement below
*   pData      - Pointer to a LCD_X_DATA structure
*/
int LCD_X_DisplayDriver(unsigned LayerIndex, unsigned Cmd, void * pData) {
    int r;
    
    switch (Cmd) 
    {
    /*
    * Required
    */
    case LCD_X_INITCONTROLLER: 
    {
        /*
        * Called during the initialization process in order to set up the
        * display controller and put it into operation. If the display
        * controller is not initialized by any external routine this needs
        * to be adapted by the customer...
        */
        _InitController();
        r = 0;
        break;
    }
    
    /* Implement all required tasks for the LCD_X_DisplayDriver function in 
    *  the following section. */
     
    default:
        r = -1;
    }
    return r;
}

/*************************** End of file ****************************/
