#include <device.h>
#include <math.h>

#include "time_psoc.h"
#include "sensors_psoc.h"
#include "i2c_communication.h"
#include "SenseAir_S8.h"
#include "sd.h"
#include "debug.h"
#include "PSoC_FatFS_Library/diskio.h" // for disk_read

// TODO: Calibration
// CO: http://www.instructables.com/id/Arduino-CO-Monitor-Using-MQ-7-Sensor/?ALLSTEPS

// https://learn.sparkfun.com/tutorials/hazardous-gas-monitor

void loadLogFromSD();

#define Vref 5.0 // This should be Vdda
static const double Vlsb = Vref/4096.0; // ADC step for a 12 bit ADC (this isn't correct, is it?)
static const double Rl = 19530.0; // Load resistance TODO: arbitrary value (it's probably way too low)
static const double Rl_CO = 21400.0; // Load resistance of CO sensor TODO: arbitrary value
static const uint8 hdcAddress = 0x40; // 100_0000

#define AIR_QUALITY_CHANNEL 0
#define NOISE_CHANNEL       1
#define CO_CHANNEL          2

static Timestamp pmTimetamp;

uint16 registers[VARIABLE_COUNT] = {}; // The n-th entry is a bit pattern that indicates if the m-th sensor measures the n-th variable
SensorStatus statuses[SENSOR_COUNT] = {[0 ... SENSOR_COUNT-1] STATUS_INITIALIZING}; // the "[0 .. SENSOR_COUNT-1]" is a designated
                                                                                    // initializer (http://stackoverflow.com/a/1065832)
float values[VARIABLE_COUNT + 2]; // Current values
struct
{
    bool8 invalid; // False if the sensor was OK the whole log cycle

    // These are to compute the average value
    float sum;
} logStatistics[VARIABLE_COUNT + 2];
bool8 updateds[arrayCount(values)] = {}; // True if the value or the status of the sensor have changed since the last read

// Converts a pair (variable, sensor) to a unique index in the values or updateds arrays
static int getFlatIndex(Variable variable, Sensor sensor)
{
    int index = 0;
    for (int var = 0; var <= variable; var++)
    {
        if ((var == variable) && (sensor == DEFAULT_SENSOR))
            return index;

        for (int s = 0; s < SENSOR_COUNT; s++)
        {
            if ((var == variable) && (s == sensor))
            {
                if (registers[var] & (1 << s))
                {
                    return index;
                }
                else
                {
                    return -1;
                }
            }

            if (registers[var] & (1 << s))
            {
                index++;
            }
        }
    }
    return -1;
}

static void regSensorAndVar(Sensor sensor, Variable variable)
{
    registers[variable] |= (1 << sensor);
}

static SensorStatus getStatus(Sensor sensor)
{
    return statuses[sensor];
}

static void setStatus(Sensor sensor, SensorStatus status)
{
    float oldStatus = statuses[sensor];
    if (oldStatus != status)
    {
        // Set the update flag for each variable the sensor measures
        for (int variable = 0; variable < VARIABLE_COUNT; variable++)
        {
            if (registers[variable] & (1 << sensor))
            {
                updateds[getFlatIndex(variable, sensor)] = true;
            }
        }
    }
    statuses[sensor] = status;
}

static void writeVariable(Sensor sensor, Variable variable, float value)
{
    int index = getFlatIndex(variable, sensor);
    float oldValue = values[index];
    if (oldValue != value)
        updateds[index] = true;
    values[index] = value;
}

void initSensors()
{
    loadLogFromSD();

    FAN_LED_PWM_Start();

    // Registering the sensors //
    regSensorAndVar(AQ_SENSOR, AQ);
    regSensorAndVar(CO_SENSOR, CO);
    regSensorAndVar(NOISE_SENSOR, NOISE);

    regSensorAndVar(IAQ, CO2);
    regSensorAndVar(IAQ, VOC);

    regSensorAndVar(S8, CO2);

    regSensorAndVar(PM_SENSOR, PM);

    regSensorAndVar(HDC1000, HUMIDITY);
    regSensorAndVar(HDC1000, TEMPERATURE);

    // Initing them //

    // Sensors that use the ADC (Air Quality, CO, noise)
    ADC_Seq_Start();
    ADC_Seq_StartConvert();

    // HDC1000
    i2cStart();
    uint8 buffer[3] = {0x02, 0x10, 0x00}; // "Please, sample temperature and humidity in sequence, using maximum resolution"
    i2cBeginTransaction();
    i2cWrite(hdcAddress, buffer, 3);
    i2cEndTransaction();

    // SenseAir S8 (CO2)
    S8_Rx_ISR_Start(); /* Inicializa la interrupcin de recepcin */
    UART_S8_Start();

    // Particulate matter sensor
    pmTimetamp = getTimestamp();

    CyGlobalIntEnable;
}

void updateIAQ()
{
    uint8 buffer[9];
    uint8 iaqAddress = 0x5a;

    i2cBeginTransaction();
    i2cRead(iaqAddress, buffer, arrayCount(buffer));
    bool8 error = i2cEndTransaction();

    uint16 co2 = (buffer[0] << 8) | buffer[1];
    uint16 voc = (buffer[7] << 8) | buffer[8];

    uint8 status = buffer[2];
    uint32 resistance = (buffer[3] << 24) | (buffer[4] << 16) | (buffer[5] << 8) | buffer[6];

    if (error || status == 0x80) // 0x80: ERROR
    {
        setStatus(IAQ, STATUS_ERROR);
    }
    else if (status == 0x10) // warming up
    {
        setStatus(IAQ, STATUS_INITIALIZING);
    }
    else if (status == 0x00) // data valid
    {
        setStatus(IAQ, 0);
        writeVariable(IAQ, CO2, (float)co2);
        writeVariable(IAQ, VOC, (float)voc);
    }
}


//
// Air quality click (MQ-135)
//

void updateAirQualitySensor()
{
    //ADC_Air_IsEndConversion(ADC_Air_RETURN_STATUS);

    int16 raw = ADC_Seq_GetResult16(AIR_QUALITY_CHANNEL); // TODO: could we get a broken value?

    // PPM (from mikroElektronika examples)
    double Vo = (double)raw * Vlsb;
    double Rs = Rl * (Vref - Vo)/Vo;               // Sensor resistance
    double ratio = Rs/Rl;                          // Calculate ratio TODO: what? why?
    double logPPM = (log10(ratio) * -0.8) + 0.9;   // Calculate ppm
    double ppm = pow(10, logPPM);                  // Calculate ppm

    writeVariable(AQ_SENSOR, AQ, ppm);
}


//
// CO click
//

void updateCOSensor()
{
    static Timestamp t; // Timestamp of the start of the last preheating, heating or cooling periods

    enum {NOT_INIT, PREHEAT, HEAT, COOL};
    static uint8 state = NOT_INIT;

    switch (state)
    {
        case NOT_INIT:
        {
            t = getTimestamp();
            setStatus(CO_SENSOR, STATUS_PREHEATING);
            state = PREHEAT;
            // turn heater on
        } break;

        case PREHEAT:
        {
            if (secondsSince(t) >= 48 * 3600) // 48 hours
            {
                t = getTimestamp();
                state = HEAT;
            }
        } break;

        case HEAT:
        {
            if (secondsSince(t) >= 60)
            {
                t = getTimestamp();
                state = COOL;
                // TODO: turn the heater off
            }
        } break;

        case COOL:
        {
            if (secondsSince(t) >= 90)
            {
                // OK, time to read the sensor

                int16 raw = ADC_Seq_GetResult16(CO_CHANNEL); // TODO: could we get a broken value?

                // Taken from the examples
                // https://libstock.mikroe.com/projects/download/1026/2719/1419930423_co_click___examp_mikroc_avr.rar
                double Vrl = (double)raw * Vlsb;
                double Rs = Rl_CO * (5 - Vrl)/Vrl;                   // Calculate sensor resistance
                double ratio = Rs/Rl_CO;                             // Calculate ratio
                double lgPPM = (log10(ratio) * -3.7) + 0.9948;    // Calculate ppm
                double ppm = pow(10, lgPPM);                      // Calculate ppm

                setStatus(CO_SENSOR, 0);
                writeVariable(CO_SENSOR, CO, ppm);

                t = getTimestamp();
                state = HEAT;
                // TODO: turn the heater on
            }
        } break;
    }
}


//
// Noise sensor click
//

void updateNoiseSensor()
{
    int16 raw = ADC_Seq_GetResult16(NOISE_CHANNEL); // TODO: could we get a broken value?
    // The maximum voltage is 3.3 V
    double sample = Vlsb * (double)raw /3.3; // sample is now in [0, 1]

    static double mean = 0;
    static double max = 0;
    static double min = 4096;
    static int count = 0;

    if (sample < min) min = sample;
    if (sample > max) max = sample;
    mean += sample;
    count++;

    double noise = sample * 2500;

    static Timestamp lastPrint = 0;
    if (msSince(lastPrint) > 500)
    {
        lastPrint = getTimestamp();
        mean /= count;
        uartPrint("Noise: %.3f. Instantaneous: %.3f. Mean: %.3f. Max: %.3f. Min: %.3f\n", noise, sample, mean, max, min);

        mean = 0;
        max = 0;
        min = 4096;
        count = 0;
    }

    setStatus(NOISE_SENSOR, 0);
    writeVariable(NOISE_SENSOR, NOISE, noise);
}


//
// HDC1000 click - Humidity and temperature
//

void updateHDC()
{
    static Timestamp lastTrigger = 0; // Timestamp of the last instant we triggered a measurement 
    static bool8 firstCall = true;

    if (firstCall)
    {
        firstCall = false;

        i2cBeginTransaction();
        i2cWriteByte(hdcAddress, 0x00);   // Triggering the next measurement
        bool8 error = i2cEndTransaction();

        lastTrigger = getTimestamp();
        if (error) 
            setStatus(HDC1000, STATUS_ERROR);
        return;
    }

    uint8 buffer[4];

    bool8 hdcDataIsReady = (msSince(lastTrigger) >= 22); // We have to wait for the ADC in the HDC1000 to finish sampling
    if (hdcDataIsReady && (secondsSince(lastTrigger) >= 1)) // Also, we don't want to update the data very often 
    {
        i2cBeginTransaction();
        i2cRead(hdcAddress, buffer, 4);   // Reading temperature and humidity registers
        i2cWriteByte(hdcAddress, 0x00);   // Triggering the next measurement
        bool8 error = i2cEndTransaction();
        lastTrigger = getTimestamp();

        if (error)
        {
            setStatus(HDC1000, STATUS_ERROR);
        }
        else
        {
            uint16 humidityRaw = (buffer[2] << 8) | buffer[3];
            float humidity = 100.0f * (float)(humidityRaw) / (1 << 16);

            uint16 temperatureRaw = (buffer[0] << 8) | buffer[1];
            float temperature = ((float)(temperatureRaw) / (1 << 16)) * 165.0 - 40.0;

            setStatus(HDC1000, 0);
            writeVariable(HDC1000, HUMIDITY, humidity);
            writeVariable(HDC1000, TEMPERATURE, temperature);
        }
    }
}


//
// SenseAir's S8 - CO2
//

void updateS8()
{
    uint16 raw = get_co2_S8();

    // TODO: When the sensor starts, it seems to return 0. This is a hack to prevent
    // the system from writing this 0.
    if (raw == 0) raw = S8_READ_INCOMPLETE;

    if (raw == S8_READ_INCOMPLETE)
        return;

    if (raw == S8_READ_ERROR)
    {
        setStatus(S8, STATUS_ERROR);
        return;
    }
    else if (getStatus(S8) & STATUS_ERROR)
    {
        setStatus(S8, STATUS_INITIALIZING);
        return;
    }

    if (getStatus(S8) & STATUS_INITIALIZING)
    {
        setStatus(S8, 0);
    }

    writeVariable(S8, CO2, (float)raw);
}


//
// Grove Dust Sensor - Particulate matter
//

// TODO: The sensor doesn't work properly in high humidity environments. Check that in here.

// (ug/m^3)
static float pmConvertToUgPerM3(long concentrationPM10)
{
    // From https://forum.mysensors.org/post/39611
    double pi = 3.14159;
    double density = 1.65e12;
    double r10 = 0.44e-6;
    double vol10 = (4/3) * pi * pow(r10, 3);
    double mass10 = density * vol10;
    double K = 3531.5;
    return (concentrationPM10) * K * mass10;
}


#define PM_TIMER_PERIOD 65536  // (see the PSoC Creator schematic)
#define PM_CLOCK_PERIOD 0.1e-3 // 10 kHz (see the PSoC Creator schematic)

void updatePMSensor()
{
    static bool8 stabilized = false;

    // After powering up the sensor, we need to wait 1 minute for it to stabilize
    if (!stabilized)
    {
        float pmStabilizationTime = 0.0; // should be 60.0f, 1 minute, according to the datasheet
        if (secondsSince(pmTimetamp) >= pmStabilizationTime)
        {
            PM_Timer_Start();
            //startCount = PM_Timer_ReadCounter();
            pmTimetamp = getTimestamp();
            stabilized = true;
        }
        else
        {
            return;
        }
    }

    // OK, it should be stabilized by now
    static int16 startCount = 0;  // number of timer counts at the beginning of the last count refresh
    static uint32 totalCount = 0; //
    static float lastUpdateTime = 0.0f;

    float timeElapsedInSeconds = secondsSince(pmTimetamp);
    if (timeElapsedInSeconds >= lastUpdateTime + 2.0f) // 2.0f is the number of seconds between totalCount refreshes
    {
        uint16 count = PM_Timer_ReadCounter();
        int32 countDiff = (int32)startCount - (int32)count;
        if (countDiff < 0) // the counter wrapped
            countDiff += PM_TIMER_PERIOD;

        totalCount += (uint32)countDiff;

        // Prepare for the next sampling
        lastUpdateTime = timeElapsedInSeconds;
        startCount = count;

        if (timeElapsedInSeconds >= 30.0f) // this should be 30.0f, I believe (according to the datasheet? it isn't clear)
        {
            setStatus(PM_SENSOR, 0);
            //if (getStatus(PM_SENSOR) & STATUS_STABILIZING)
                //setStatus(PM_SENSOR, 0);

            float highTime = (float)totalCount * PM_CLOCK_PERIOD;

            float lowOccupancy = 1.0f - highTime/timeElapsedInSeconds;
            if (lowOccupancy < 0.0f) lowOccupancy = 0;      // (could happen)
            if (lowOccupancy > 1.0f) lowOccupancy = 1.0f;

            double ratio = lowOccupancy * 100.0;

            // using spec sheet curve (taken from http://wiki.seeed.cc/Grove-Dust_Sensor/#demos)
            double pmInWeirdUnits = 1.1*pow(ratio,3)-3.8*pow(ratio,2)+520*ratio+0.62; // pcs/0.01cf (pcs: particles, I guess | cf: cubic foot, 0.01cf = 283 ml)
            double pm = pmConvertToUgPerM3(pmInWeirdUnits);

            writeVariable(PM_SENSOR, PM, pm);

            // Reset everything for the next iteration
            pmTimetamp = getTimestamp();
            lastUpdateTime = 0;
            startCount = PM_Timer_ReadCounter();
            totalCount = 0;
        }
    }
}

// Finds what the default sensor is
Sensor getDefaultSensor(Variable variable)
{
    Sensor sensor = DEFAULT_SENSOR;

    uint16 reg = registers[variable];
    for (int _sensor = 0; _sensor < SENSOR_COUNT; _sensor++)
    {
        if (reg & (1 << _sensor))
        {
            // This is the first sensor that measures this variable 
            sensor = _sensor;
            break;
        }
    }

    return sensor;
}

VariableData getVariableFromSensor(Variable variable, Sensor sensor)
{
    VariableData data = {};

    if (sensor == DEFAULT_SENSOR)
    {
        sensor = getDefaultSensor(variable);
    }

    int index = getFlatIndex(variable, sensor);
    data.value = values[index];
    data.updated = updateds[index];
    data.status = statuses[sensor];
    // TODO: NO! It should be cleared externally (maybe in updateSensors)
    //updateds[index] = false;

    return data;
}

// From the default sensor
VariableData getVariable(Variable variable)
{
    return getVariableFromSensor(variable, DEFAULT_SENSOR);
}

// Log memory map
//       0-3  new counter to update
//         4  set if we have finished a write and are moving the new counter to the current counter bytes
//       5-8  current counter value
//  512-2559  timestamps
//    2560-*  variables
#define NEWINDEX_BASE     0UL
#define STATUS_BASE       4UL // operation in progress (see below)
#define INDEX_BASE        5UL // last valid log index
#define TIMESTAMP_BASE  512UL // to remove
#define VARIABLES_BASE 2560UL

typedef enum
{
    NOP            = 0,
    INDEX_WRITE_OP = 1,
    CLEAR_OP       = 2,
} LogOperation;

#define MAX_LOG_SIZE_PER_VARIABLE 2560UL

#define LOG_COUNT (24*12) // number of samples (24 hours, 12 samples per hour)
float loggedValues[arrayCount(values)][LOG_COUNT]; // min, average, max (NAN if the reading was invalid - because the sensor wasn't ready, for example)
uint32 logIndex = 0; // current sample index (0 means 00h, 1 means 00h + 5 min)
int logPeriodInSeconds = 5 * 60; // 5 minutes
//int logPeriodInSeconds = 1; // each second (for debugging)

void commitLog(uint32 index)
{
    static uint8 sectorBuffer[512];
    
    // TODO: Can the SD reorder the operations? I'm assuming it writes a lower address before a higher one.

    *((uint32 *)(sectorBuffer + NEWINDEX_BASE)) = index;
    *((uint8  *)(sectorBuffer + STATUS_BASE))   = INDEX_WRITE_OP;
    *((uint32 *)(sectorBuffer + INDEX_BASE))    = index;
    sdWriteSector(0, sectorBuffer);

    *((uint8 *)(sectorBuffer + STATUS_BASE)) = NOP;
    sdWriteSector(0, sectorBuffer);
}

void loadLogFromSD()
{
    uint8 status = NOP;
    uint32 index = 0;
    uint32 newIndex = 0;

    bool fail = false;
    fail = fail || !sdRead(STATUS_BASE, &status, 1);
    fail = fail || !sdRead(INDEX_BASE, (uint8*)&index, 4);
    fail = fail || !sdRead(NEWINDEX_BASE, (uint8*)&newIndex, 4);

    if (fail)
    {
        uartPrint("Loading log from SD failed!\n");
    }

    if (status == INDEX_WRITE_OP) // we didn't finished moving newIndex to index; we'll do that now
    {
        index = newIndex;
        sdWrite(INDEX_BASE, (uint8*)&index, 4);

        status = NOP;
        sdWrite(STATUS_BASE, (uint8*)&status, 1);
    }
    else if (status == CLEAR_OP) // we didn't finished clearing all data
    {
        for (int i = 0; i < arrayCount(loggedValues); i++)
        {
            for (int j = 0; j < arrayCount(loggedValues[i]); j++)
            {
                loggedValues[i][j] = NAN;
            }
        }

        sdWrite(VARIABLES_BASE, (uint8*)loggedValues, sizeof(loggedValues));

        status = NOP;
        sdWrite(STATUS_BASE, (uint8*)&status, 1);
    }

    logIndex = index;
    sdRead(VARIABLES_BASE, (uint8*)loggedValues, sizeof(loggedValues));
}

void updateSensors()
{
    updateHDC();
    updateCOSensor();
    updateS8();
    updateIAQ();
    updateNoiseSensor();
    updatePMSensor();
    updateAirQualitySensor();

    static int statisticsCount = 0;

    for (int variable = 0; variable < VARIABLE_COUNT; variable++)
    {
        for (int sensor = 0; sensor < SENSOR_COUNT; sensor++)
        {
            int index = getFlatIndex(variable, sensor);
            if (index >= 0)
            {
                logStatistics[index].invalid |= (statuses[sensor] != 0);

                if (!logStatistics[index].invalid)
                {
                    logStatistics[index].sum += values[index];
                }
            }
        }
    }

    statisticsCount++;

    // Logging
    static Timestamp lastLog = 0;
    if (secondsSince(lastLog) >= logPeriodInSeconds)
    {
        Timestamp debugT0 = getTimestamp();

        lastLog = getTimestamp();

        for (int variable = 0; variable < VARIABLE_COUNT; variable++)
        {
            for (int sensor = 0; sensor < SENSOR_COUNT; sensor++)
            {
                int index = getFlatIndex(variable, sensor);
                if (index >= 0)
                {
                    bool isValid = !logStatistics[index].invalid;

                    loggedValues[index][logIndex] = isValid ? logStatistics[index].sum/statisticsCount : NAN;

                    logStatistics[index].invalid = false;
                    logStatistics[index].sum = 0;

                    uint64 address = VARIABLES_BASE + index * sizeof(loggedValues[0]);
                    address += logIndex * sizeof(float);
                    Timestamp t0 = getTimestamp();
                    //sdWrite(address, (uint8*)&loggedValues[index][logIndex], sizeof(float));
                    //sdWrite(address, (uint8*)&address, 1);
                    //CMD0
                    static uint8 buffer[512];
                    //sdRead(0, buffer, 512);
                    disk_read(0, buffer, 0, 1);
                    uartPrint("index: %d %f\n", index, secondsSince(t0));
                }
            }
        }

        //sdWrite(VARIABLES_BASE, (uint8*)loggedValues, sizeof(loggedValues));

        logIndex++;
        if (logIndex >= LOG_COUNT)
            logIndex = 0;

        //commitLog(logIndex);

        statisticsCount = 0;

        uartPrint("sizeof(loggedValues): %u\n", sizeof(loggedValues));
        uartPrint("logging time: %f\n", secondsSince(debugT0));
    }
}

void clearLog()
{
    uint8 status = CLEAR_OP;
    sdWrite(STATUS_BASE, (uint8*)&status, 1);

    for (int i = 0; i < arrayCount(loggedValues); i++)
    {
        for (int j = 0; j < arrayCount(loggedValues[i]); j++)
        {
            loggedValues[i][j] = NAN;
        }
    }

    sdWrite(VARIABLES_BASE, (uint8*)loggedValues, sizeof(loggedValues));

    status = NOP;
    sdWrite(STATUS_BASE, (uint8*)&status, 1);
}

void readLog(float **log, uint32 *count, uint32 *currentIndex, Variable variable, Sensor sensor, uint32 seconds)
{
    *currentIndex = logIndex;
    *count = arrayCount(loggedValues[0]);
    *log = loggedValues[getFlatIndex(variable, sensor)];
}

void setFanSpeed(uint8 speed)
{
    FAN_LED_PWM_WriteCompare(speed);
}

uint8 getFanSpeed()
{
    return FAN_LED_PWM_ReadCompare();
}
